@@ -13,6 +13,11 @@ class Filter:
         pass
 
 
+class NoneFilter(Filter):
+    code = '(none)'
+    name = '(aucun)'
+
+
 def filters_dictionary(filters):
     return {item.code: item for item in filters}
 
