@@ -11,9 +11,7 @@ from ..models import Column
 class ColumnTables(Table):
     def __init__(self, rows):
         super().__init__(rows, [
-            TableColumn('label', "Libellé"),
-            TableColumn('criteria', "Critère"),
-            TableColumn('value', "Valeur")
+            TableColumn('label', "Libellé")
         ])
         self.update = 'django:column_update'
         self.create = 'django:column_create'
@@ -32,6 +30,9 @@ def column_list(request):
     if len(search) > 0:
         columns = Column.objects.filter(label=search)
     context['search'] = search
+
+    columns = columns.order_by('order')
+
     context['table'] = ColumnTables(paginate(request, context, columns))
 
     return render(request, 'columns.html', context)
