@@ -9,7 +9,7 @@ from django.shortcuts import render
 from django.urls import reverse
 from django.utils.translation import gettext as _
 from foledol.django.logger import log
-from foledol.django.models import Grid, ConditionGroup, Condition
+from foledol.django.models import Grid, ConditionGroup, Condition, ORDER_BY_ASC, ORDER_BY_DESC
 from foledol.django.reports.grid import GridReport
 from foledol.django.templatetags.form_extras import value
 from foledol.django.tools.field import TextField, BooleanField
@@ -45,6 +45,7 @@ class ColumnSubTable(Table):
     def __init__(self, rows, read_only=False):
         super().__init__(rows, [
             TableColumn('label', "Libellé", method='label_as_str'),
+            TableColumn('order_by', "Tri", method='order_by_as_str'),
             TableColumn('order', "Ordre")
         ])
         self.heading = 'Colonnes'
@@ -65,6 +66,15 @@ UPDATE_ACTIONS = [
 ]
 
 
+def grid_get_filter(code, grid_filters):
+    if code == '(none)':
+        return None
+    for grid_filter in grid_filters:
+        if grid_filter.code == code:
+            return grid_filter
+    return None
+
+
 def grid_form(request, grid, read_only=False):
     if not request.user.is_staff:
         return HttpResponseForbidden()
@@ -73,9 +83,15 @@ def grid_form(request, grid, read_only=False):
     context['base'] = settings.DEFAULT_SPACE + '/base.html'
 
     action = get_path(request, context, grid, 'grid', space='django')
+    if action == 'clone':
+        grid = grid.clone()
+        request.session['path'] = request.POST['path']
+        return HttpResponseRedirect(reverse('django:grid_update', kwargs={'pk': grid.id}) + '?back')
 
     context['read_only'] = read_only
 
+    get_param(request, context, 'grid_sort', '')
+
     edit_as_str = get_param(request, context, 'edit', 'False')
     edit = edit_as_str != 'False'
 
@@ -84,12 +100,12 @@ def grid_form(request, grid, read_only=False):
     context['grid_url'] = reverse('django:grid_update', kwargs={'pk': grid.id}) if grid else None
 
     form = GridForm(context)
-    form.read(request.POST, grid)
-    if action == 'update':
+    form.read(request.POST if action else {}, grid)
+    #if action == 'update':
         # TODO: fix the issue on BooleanField (Read)
-        form.field('distinct').value = 'distinct' in request.POST
-        form.field('group_by').value = 'group_by' in request.POST
-        form.field('show_on_home').value = 'show_on_home' in request.POST
+        #form.field('distinct').value = 'distinct' in request.POST
+        #form.field('group_by').value = 'group_by' in request.POST
+        #form.field('show_on_home').value = 'show_on_home' in request.POST
 
     if len(action) > 0:
         form.validate()
@@ -98,11 +114,20 @@ def grid_form(request, grid, read_only=False):
     context['table'] = grid.table if grid else 'member'
 
     columns = grid.column_set.all().order_by('order') if grid else None
+
     context['columns'] = columns
     context['table_columns'] = ColumnSubTable(columns, read_only=not request.user.is_staff)
 
     grid_table = settings.GRID_TABLES[grid.table] if grid and grid.table in settings.GRID_TABLES else None
 
+    grid_filter = grid.filter if grid and grid.filter else '(none)'
+    grid_filter = get_param(request, context, 'filter', grid_filter)
+    if grid:
+        grid.filter = grid_filter
+
+    grid_filters = grid_table.filters if grid_table else []
+    context['grid_filters'] = grid_filters
+
     if grid and not grid.group_by:
         if grid_table and len(grid_table.reports) > 0:
             context['reports'] = grid_table.reports
@@ -119,7 +144,12 @@ def grid_form(request, grid, read_only=False):
         edit = False
     context['edit'] = edit
 
-    objects = get_objects(grid, grid_table, context) if grid else None
+    if action == 'update':
+        form.save(grid)
+
+    grid_filter = grid_get_filter(grid_filter, grid_filters)
+
+    objects = get_objects(grid, grid_table, grid_filter, context) if grid else None
 
     context['update'] = settings.DEFAULT_SPACE + ':' + grid.table + '_update' if grid else None
 
@@ -156,15 +186,31 @@ def grid_form(request, grid, read_only=False):
     context['condition_group'] = grid.condition_group if grid else None
     context['condition_as_str'] = grid.condition_group.as_str() if grid else None
 
-    rows = grid_rows(grid, objects, fields) if objects else None
+    try:
+        rows = grid_rows(grid, objects, fields) if objects else None
+    except Exception as ex:
+        context['warning'] = str(ex)
+        rows = None
+
+    sort_columns = []
+    sort_columns_asc = []
+    sort_columns_desc = []
 
     if columns:
 
         default_sort = None
         for column in columns:
-            if not column.hidden:
+            if default_sort is None:
                 default_sort = column.label + "_asc"
-                break
+            if column.order_by == ORDER_BY_ASC:
+                sort_columns.append(column.label)
+                sort_columns_asc.append(column.label)
+            if column.order_by == ORDER_BY_DESC:
+                sort_columns.append('-' + column.label)
+                sort_columns_desc.append(column.label)
+        if rows and len(sort_columns) > 0:
+            rows = rows.order_by(*sort_columns)
+
         full_name = "grid_" + str(grid.id) + ".sort"
         default_sort = request.session[full_name] if full_name in request.session else default_sort
 
@@ -175,13 +221,17 @@ def grid_form(request, grid, read_only=False):
 
         request.session[full_name] = context['sort'] = sort
 
-        if sort:
+        if rows and sort and len(sort_columns) == 0:
             if sort.endswith("_asc"):
                 sort = sort[0:-4]
             if sort.endswith("_desc"):
                 sort = "-" + sort[0:-5]
             rows = rows.order_by(sort)
 
+    context['sort_columns'] = sort_columns
+    context['sort_columns_asc'] = sort_columns_asc
+    context['sort_columns_desc'] = sort_columns_desc
+
     if 'copy' in request.GET:
         data = grid_copy(grid, rows, fields)
         return HttpResponse(data, 'application/json')
@@ -199,6 +249,8 @@ def grid_form(request, grid, read_only=False):
         for report_item in grid_table.reports:
             if report_item.key == key and report_item.report:
                 report = report_item.report
+                if hasattr(report, 'title'):
+                    report.title = grid.comment if grid.comment else grid.name
                 output = report.output if hasattr(report, 'output') else grid.table + ".pdf"
                 return print_report(report.build(rows), output)
 
@@ -264,19 +316,27 @@ def grid_view(request, pk):
     return grid_form(request, grid, True) if grid else error(request)
 
 
-def get_objects_from_manager(grid, context, manager, table):
+def get_objects_from_manager(grid, context, manager, table, grid_filter):
     if grid.filter_by:
         try:
             sql = "SELECT id FROM {table} WHERE {filter_by}".format(table=table, filter_by=grid.filter_by)
-            raw_sql = RawSQLite(sql, ()) if settings.CREATE_INSCRIPTION else RawSQL(sql, ())
+
+            sql_lite = False
+            if 'default' in settings.DATABASES:
+                default = settings.DATABASES['default']
+                if 'ENGINE' in default:
+                    sql_lite = default['ENGINE'] == 'django.db.backends.sqlite3'
+
+            raw_sql = RawSQLite(sql, ()) if sql_lite else RawSQL(sql, ())
             return manager.filter(id__in=raw_sql)
         except Exception as ex:
             context['warning'] = ex
-    return manager.all()
+    objects = manager.all()
+    return grid_filter.filter(objects) if grid_filter else objects
 
 
-def get_objects(grid, grid_table, context):
-    return get_objects_from_manager(grid, context, grid_table.manager, grid_table.table)
+def get_objects(grid, grid_table, grid_filter, context):
+    return get_objects_from_manager(grid, context, grid_table.manager, grid_table.table, grid_filter)
 
 
 def grid_condition_group_add(condition_group, type):
@@ -331,7 +391,7 @@ def grid_rows(grid, rows, fields):
         values = []
         for column in grid.column_set.all():
             field = get_field(column.label, fields)
-            if field and not column.hidden:
+            if field:
                 values.append(field.key)
         rows = rows.values(*values)
 
@@ -408,7 +468,7 @@ def grid_copy(grid, rows, fields):
 
     for column in grid.column_set.all().order_by('order'):
         field = get_field(column.label, fields)
-        if field and not column.hidden:
+        if field:
             if len(data) > 0:
                 data += "\t"
             data += field.label
@@ -418,7 +478,7 @@ def grid_copy(grid, rows, fields):
         line = ""
         for column in grid.column_set.all():
             field = get_field(column.label, fields)
-            if field and not column.hidden:
+            if field:
                 if len(line) > 0:
                     line += "\t"
                 line += str(get_value(grid, row, column))
@@ -443,7 +503,7 @@ def grid_export(grid, rows, fields):
     i = 0
     for column in grid.column_set.all().order_by('order'):
         field = get_field(column.label, fields)
-        if field and not column.hidden:
+        if field:
             worksheet.write(0, i, field.label, column_header)
             if column.width and column.width > 0:
                 worksheet.set_column(i, i, round(column.width / 10))
@@ -454,7 +514,7 @@ def grid_export(grid, rows, fields):
         i = 0
         for column in grid.column_set.all():
             field = get_field(column.label, fields)
-            if field and not column.hidden:
+            if field:
                 worksheet.write(j, i, get_value(grid, row, column))
                 i += 1
         j += 1
